'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _babelTraverse = require('babel-traverse');

var _babelTraverse2 = _interopRequireDefault(_babelTraverse);

var _babelTypes = require('babel-types');

var _lodash = require('lodash');

var _pascalCase = require('pascal-case');

var _pascalCase2 = _interopRequireDefault(_pascalCase);

var _getImports = require('./get-imports');

var _getImports2 = _interopRequireDefault(_getImports);

var _normalizeTagName = require('./normalize-tag-name');

var _normalizeTagName2 = _interopRequireDefault(_normalizeTagName);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Get names of implicit dependencies for component ast
 * @param {Object} ast - ast to search for unbound identifiers
 * @return {Array<String>} local names to import implicitly
 * @deprecate
 */
exports.default = ast => {
	const unboundIdentifiers = [];

	(0, _babelTraverse2.default)(ast, {
		ReferencedIdentifier: function ReferencedIdentifier(path) {
			const binding = path.scope.getBinding(path.node.name);

			if (!binding) {
				if (path.isJSXIdentifier()) {
					const normalizedName = (0, _normalizeTagName2.default)(path.node.name);

					if (!_babelTypes.react.isCompatTag(normalizedName)) {
						unboundIdentifiers.push(normalizedName);
						return;
					}
				}
			}
		}
	});

	const explicitDependencies = (0, _getImports2.default)(ast).identifiers.map(_ref => {
		let value = _ref.value;
		return (0, _pascalCase2.default)(value);
	});

	const implicitDependencies = (0, _lodash.uniq)((0, _lodash.difference)(unboundIdentifiers, explicitDependencies));

	return implicitDependencies.reduce((registry, implicitDependencyName) => {
		return _extends({}, registry, {
			[implicitDependencyName]: (0, _lodash.kebabCase)(implicitDependencyName)
		});
	}, {});
};

module.change_code = 1; // eslint-disable-line

module.exports = exports['default'];