'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _babelTraverse = require('babel-traverse');

var _babelTraverse2 = _interopRequireDefault(_babelTraverse);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) arr2[i] = arr[i]; return arr2; } else { return Array.from(arr); } }

/**
 * Check if a given plain jsx is compatible with transformation to a stateless
 * component
 * - no collision with "props"
 * - no collision with "context"
 *
 * @type {object} ast
 * @return boolean
 */
exports.default = ast => {
	let collisions = [];

	(0, _babelTraverse2.default)(ast, {
		Declaration: function Declaration(path) {
			if (path.parent.type !== 'Program') {
				return;
			}

			const declarations = path.node.declarations || [path.node];

			const colliding = declarations
			// Check for naming collision
			.filter(decl => {
				return decl.id && ['props', 'context'].includes(decl.id.name);
			});

			collisions = [].concat(_toConsumableArray(collisions), _toConsumableArray(colliding));
		}
	});

	return collisions.length === 0;
};

module.change_code = 1; // eslint-disable-line

module.exports = exports['default'];